/// <reference types="node" />
import { EventEmitter } from 'events';
export interface IEventData {
    caller: {
        chain_id: string;
        local: string;
    };
    address: {
        chain_id: string;
        local: string;
    };
    block_height: string;
    encoded_body: string;
}
export interface IJSONRPCError {
    code: number;
    message: string;
    data: any;
}
export interface IJSONRPCEvent {
    error?: IJSONRPCError;
    result?: IEventData;
}
export declare type WSRPCClientEventListener = (event: IJSONRPCEvent) => void;
export declare enum WSRPCClientEvent {
    /** Emitted when a connection is established with the server. */
    Connected = "connected",
    /** Emitted when the connection to the server is closed down. */
    Disconnected = "disconnected",
    /** Emitted when an error is encountered that can't be propagated in a more sensible fashion. */
    Error = "error",
    /** Emitted when an event message (not a response message) is received from the server. */
    Message = "message",
    /**
     * Emitted when chain events subscription status changes.
     * Listener will receive a single boolean value, `true` indicates that the event subscription is
     * active, `false` indicates that it's inactive.
     */
    Subscribed = "subscribed",
}
/**
 * Sends JSON-RPC messages via web sockets.
 */
export declare class WSRPCClient extends EventEmitter {
    url: string;
    private _client;
    private _rpcId;
    private _isSubcribed;
    private _getNextRequestId;
    requestTimeout: number;
    readonly isSubscribed: boolean;
    /**
     *
     * @param url
     * @param opts Options object
     * @param opts.requestTimeout Number of milliseconds to wait for a network operation to complete.
     * @param opts.reconnectInterval Number of milliseconds to wait before attempting to reconnect
     *                               (in case the connection drops out).
     * @param opts.maxReconnects Maximum number of times to reconnect, defaults to infinity.
     */
    constructor(url: string, opts?: {
        requestTimeout?: number;
        reconnectInterval?: number;
        maxReconnects?: number;
        generateRequestId?: (method: string, params: object | any[]) => string;
    });
    /**
     * Gracefully closes the underlying web socket connection.
     */
    disconnect(): void;
    /**
     * Waits for a connection to be established to the server (if it isn't already).
     * @returns A promise that will be resolved when a connection is established.
     */
    ensureConnectionAsync(): Promise<void>;
    /**
     * Sends a JSON-RPC message.
     * @param method RPC method name.
     * @param params Parameter object or array.
     * @returns A promise that will be resolved with the value of the result field (if any) in the
     *          JSON-RPC response message.
     */
    sendAsync<T>(method: string, params: object | any[]): Promise<T>;
    private _onEventMessage;
}
